/*
LCD drive
	Pin assignment
	13:RA0:O:LCD E
	11:RA2:O:LCD RS
	10:RC0:O:LCD D4
	 9:RC1:0:LCD D5
	 8:RC2:O:LCD D6
	 7:RC3:O:LCD D7
	LCD R/W is only VSS
	DB0 to DB3 are must be open
 */

//#include	<xc.h>
#include	"main.h"	//Fix typo ***1.50
#include	"lcd.h"

//This declaration was 	obsoleteness.  ***1.50
//static bit LCD_RS @ ((unsigned) &PORTA * 8 + 2); // Register select (RA2)
//static bit LCD_EN @ ((unsigned) &PORTA * 8 + 0); // Enable (RA0) 


uint8_t lcd_loccnt = 0;

// write a byte to the LCD in 4 bit mode

void lcd_data_byteset(uint8_t c) {
	LCD_UsePort = (uint8_t)((LCD_UsePort & LCD_PortMask) | (c & 0x0f)); //Only low 4bit DATA
	LCD_STROBE;
}

void lcd_data_set(uint8_t c) {
	LCD_UsePort =  (uint8_t)((LCD_UsePort & LCD_PortMask) | ((c >> 4) & 0x0f)); //Hi SIDE DATA
	LCD_STROBE;
	LCD_UsePort = (uint8_t)((LCD_UsePort & LCD_PortMask) | (c & 0x0f)); //low SIDE DATA
	LCD_STROBE;
}

void
lcd_write(uint8_t c) {
	LCD_RS = 0;
	LCD_UsePort = (uint8_t)((LCD_UsePort & LCD_PortMask) | ((c >> 4) &0x0f)); //Hi SIDE DATA
	LCD_STROBE;
	LCD_UsePort = (uint8_t)((LCD_UsePort & LCD_PortMask) | (c & 0x0f)); //low SIDE DATA
	LCD_STROBE;
	__delay_us(40);
}


/*
 * 	Clear and home the LCD
 */

void
lcd_clear(void) {
	lcd_write(0x1);
	lcd_loccnt = 0;
	__delay_ms(2); //PICC Pro
}

//write a string of chars to the LCD

void
lcd_puts(const char * s) {
	LCD_RS = 1; // write characters
	while (*s)
		lcd_putch(*s++);
}

//write one character to the LCD and auto increment location counter*/

void
lcd_putch(uint8_t c) {
	LCD_RS = 1; // write characters
	lcd_data_set(c);
	__delay_us(40);

	lcd_loccnt++;
	if (lcd_loccnt == LCD_LineLength1) {
		lcd_loccnt = LCD_LineNumber2;
		lcd_goto(lcd_loccnt);
	}
	if (lcd_loccnt == LCD_LineLength2) {
		lcd_loccnt = LCD_LineNumber1;
		lcd_goto(lcd_loccnt);
	}
}

//write to CG RAM 

void lcd_cgwrite(uint8_t c) {
	LCD_RS = 1; // write characters
	lcd_data_set(c);
	__delay_us(40);
}

//Go to the specified position

void lcd_goto(uint8_t pos) {
	lcd_write((uint8_t)(0x80 + pos));
	lcd_loccnt = pos;
}

//initialise the LCD - put into 4 bit mode

void
lcd_init(void) {
	LCD_RS = 0; // write control bytes
	__delay_ms(15); // power on delay (PICC Pro)
	lcd_data_byteset(0x03); // attention!
	__delay_ms(5); //wait
	LCD_STROBE; //strobe again
	__delay_us(100); //wait
	LCD_STROBE; //strobe again
	__delay_ms(5); //wait
	lcd_data_byteset(0x02); // set 4 bit mode
	__delay_us(40); //wait
	lcd_write(0x28); // 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08); // display off
	lcd_write(0x0F); // display on, blink curson on
	lcd_write(0x06); // entry mode
	lcd_loccnt = 0;
}

//Newline processing

void lcd_newline(uint8_t mode) {
	uint8_t temp;
	switch (mode) {
		case 1://the following are deleted
			if (lcd_loccnt <= LCD_LineLength1) {
				lcd_space_full((uint8_t)(LCD_LineLength1 - lcd_loccnt));
				return;
			} else if (lcd_loccnt <= LCD_LineLength2) {
				lcd_space_full((uint8_t)(LCD_LineLength2 - lcd_loccnt));
				return;
			} else return;

		default://toggle the line
			if (lcd_loccnt <= LCD_LineLength1) lcd_goto(LCD_LineNumber2);
			else if (lcd_loccnt <= LCD_LineLength2) lcd_goto(LCD_LineNumber1);
	}
}

void lcd_space_full(uint8_t count) {//spacing
	while (count--) {
		lcd_putch(' '); //space character write
	};
}

